COUNTRY_TO_CONTINENT = {
        'afghanistan': 'Asia',
        'albania': 'Europe',
        'algeria': 'Africa',
        'american samoa': 'Oceania',
        'andorra': 'Europe',
        'angola': 'Africa',
        'anguilla': 'North America',
        'antigua and barbuda': 'North America',
        'argentina': 'South America',
        'armenia': 'Asia',
        'aruba': 'North America',
        'australia': 'Oceania',
        'austria': 'Europe',
        'azerbaijan': 'Asia',
        'bahamas': 'North America',
        'bahrain': 'Asia',
        'bangladesh': 'Asia',
        'barbados': 'North America',
        'belarus': 'Europe',
        'belgium': 'Europe',
        'belize': 'North America',
        'benin': 'Africa',
        'bermuda': 'North America',
        'bhutan': 'Asia',
        'bolivia, plurinational state of': 'South America',
        'bolivia': 'South America',
        'bonaire, sint eustatius and saba': 'North America',
        'bonaire': 'North America',
        'sint eustatius': 'North America',
        'saba': 'North America',
        'bosnia and herzegovina': 'Europe',
        'botswana': 'Africa',
        'brazil': 'South America',
        'british indian ocean territory': 'Asia',
        'brunei darussalam': 'Asia',
        'bulgaria': 'Europe',
        'burkina faso': 'Africa',
        'burundi': 'Africa',
        'cabo verde': 'Africa',
        'cambodia': 'Asia',
        'cameroon': 'Africa',
        'canada': 'North America',
        'cayman islands': 'North America',
        'cayman island': 'North America',
        'central african republic': 'Africa',
        'chad': 'Africa',
        'chile': 'South America',
        'china': 'Asia',
        'christmas island': 'Asia',
        'cocos (keeling) islands': 'Asia',
        'cocos keeling islands': 'Asia',
        'cocos islands': 'Asia',
        'colombia': 'South America',
        'comoros': 'Africa',
        'congo': 'Africa',
        'cook islands': 'Oceania',
        'costa rica': 'North America',
        'croatia': 'Europe',
        'cuba': 'North America',
        'curaçao': 'North America',
        'curacao': 'North America',
        'cyprus': 'Asia',
        'czechia': 'Europe',
        "côte d'ivoire": 'Africa',
        'denmark': 'Europe',
        'djibouti': 'Africa',
        'dominica': 'North America',
        'dominican republic': 'North America',
        'ecuador': 'South America',
        'egypt': 'Africa',
        'el salvador': 'North America',
        'equatorial guinea': 'Africa',
        'eritrea': 'Africa',
        'estonia': 'Europe',
        'eswatini': 'Africa',
        'ethiopia': 'Africa',
        'falkland islands (malvinas)': 'South America',
        'faroe islands': 'Europe',
        'fiji': 'Oceania',
        'finland': 'Europe',
        'france': 'Europe',
        'french guiana': 'South America',
        'french polynesia': 'Oceania',
        'gabon': 'Africa',
        'gambia': 'Africa',
        'georgia': 'Asia',
        'germany': 'Europe',
        'ghana': 'Africa',
        'gibraltar': 'Europe',
        'greece': 'Europe',
        'greenland': 'North America',
        'grenada': 'North America',
        'guadeloupe': 'North America',
        'guam': 'Oceania',
        'guatemala': 'North America',
        'guernsey': 'Europe',
        'guinea': 'Africa',
        'guinea-bissau': 'Africa',
        'guyana': 'South America',
        'haiti': 'North America',
        'honduras': 'North America',
        'hong kong': 'Asia',
        'hungary': 'Europe',
        'iceland': 'Europe',
        'india': 'Asia',
        'indonesia': 'Asia',
        'iran, islamic republic of': 'Asia',
        'iran': 'Asia',
        'repulic of iran': 'Asia',
        'iraq': 'Asia',
        'ireland': 'Europe',
        'isle of man': 'Europe',
        'israel': 'Asia',
        'italy': 'Europe',
        'jamaica': 'North America',
        'japan': 'Asia',
        'jersey': 'Europe',
        'jordan': 'Asia',
        'kazakhstan': 'Asia',
        'kenya': 'Africa',
        'kiribati': 'Oceania',
        "korea, democratic people's republic of": 'Asia',
        "democratic people's republic of korea": 'Asia',
        'korea, republic of': 'Asia',
        'korea': 'Asia',
        'republic of korea': 'Asia',
        'kuwait': 'Asia',
        'kyrgyzstan': 'Asia',
        "lao people's democratic republic": 'Asia',
        'latvia': 'Europe',
        'lebanon': 'Asia',
        'lesotho': 'Africa',
        'liberia': 'Africa',
        'libya': 'Africa',
        'liechtenstein': 'Europe',
        'lithuania': 'Europe',
        'luxembourg': 'Europe',
        'macao': 'Asia',
        'madagascar': 'Africa',
        'malawi': 'Africa',
        'malaysia': 'Asia',
        'maldives': 'Asia',
        'mali': 'Africa',
        'malta': 'Europe',
        'marshall islands': 'Oceania',
        'martinique': 'North America',
        'mauritania': 'Africa',
        'mauritius': 'Africa',
        'mayotte': 'Africa',
        'mexico': 'North America',
        'micronesia, federated states of': 'Oceania',
        'micronesia': 'Oceania',
        'moldova, republic of': 'Europe',
        'moldova': 'Europe',
        'monaco': 'Europe',
        'mongolia': 'Asia',
        'montenegro': 'Europe',
        'montserrat': 'North America',
        'morocco': 'Africa',
        'mozambique': 'Africa',
        'myanmar': 'Asia',
        'namibia': 'Africa',
        'nauru': 'Oceania',
        'nepal': 'Asia',
        'netherlands': 'Europe',
        'new caledonia': 'Oceania',
        'new zealand': 'Oceania',
        'nicaragua': 'North America',
        'niger': 'Africa',
        'nigeria': 'Africa',
        'niue': 'Oceania',
        'norfolk island': 'Oceania',
        'north macedonia': 'Europe',
        'northern mariana islands': 'Oceania',
        'norway': 'Europe',
        'oman': 'Asia',
        'pakistan': 'Asia',
        'palau': 'Oceania',
        'palestine, state of': 'Asia',
        'palestine': 'Asia',
        'panama': 'North America',
        'papua new guinea': 'Oceania',
        'paraguay': 'South America',
        'peru': 'South America',
        'philippines': 'Asia',
        'poland': 'Europe',
        'portugal': 'Europe',
        'puerto rico': 'North America',
        'qatar': 'Asia',
        'romania': 'Europe',
        'russian federation': 'Europe',
        'rwanda': 'Africa',
        'réunion': 'Africa',
        'saint barthélemy': 'North America',
        'saint helena, ascension and tristan da cunha': 'Africa',
        'saint kitts and nevis': 'North America',
        'saint lucia': 'North America',
        'saint martin (french part)': 'North America',
        'saint pierre and miquelon': 'North America',
        'saint vincent and the grenadines': 'North America',
        'samoa': 'Oceania',
        'san marino': 'Europe',
        'sao tome and principe': 'Africa',
        'saudi arabia': 'Asia',
        'senegal': 'Africa',
        'serbia': 'Europe',
        'seychelles': 'Africa',
        'sierra leone': 'Africa',
        'singapore': 'Asia',
        'slovakia': 'Europe',
        'slovenia': 'Europe',
        'solomon islands': 'Oceania',
        'somalia': 'Africa',
        'south africa': 'Africa',
        'south georgia and the south sandwich islands': 'South America',
        'south sudan': 'Africa',
        'spain': 'Europe',
        'sri lanka': 'Asia',
        'sudan': 'Africa',
        'suriname': 'South America',
        'svalbard and jan mayen': 'Europe',
        'sweden': 'Europe',
        'switzerland': 'Europe',
        'syrian arab republic': 'Asia',
        'taiwan, province of china': 'Asia',
        'taiwan': 'Asia',
        'tajikistan': 'Asia',
        'tanzania, united republic of': 'Africa',
        'tanzania': 'Africa',
        'thailand': 'Asia',
        'togo': 'Africa',
        'tokelau': 'Oceania',
        'tonga': 'Oceania',
        'trinidad and tobago': 'North America',
        'tunisia': 'Africa',
        'turkey': 'Asia',
        'turkmenistan': 'Asia',
        'turks and caicos islands': 'North America',
        'tuvalu': 'Oceania',
        'uganda': 'Africa',
        'ukraine': 'Europe',
        'united arab emirates': 'Asia',
        'united kingdom': 'Europe',
        'united states': 'North America',
        'uruguay': 'South America',
        'uzbekistan': 'Asia',
        'vanuatu': 'Oceania',
        'venezuela, bolivarian republic of': 'South America',
        'venezuela': 'South America',
        'viet nam': 'Asia',
        'vietnam': 'Asia',
        'virgin islands, british': 'North America',
        'virgin islands, u.s.': 'North America',
        'wallis and futuna': 'Oceania',
        'yemen': 'Asia',
        'zambia': 'Africa',
        'zimbabwe': 'Africa',
        'åland islands': 'Europe',
        'aland islands': 'Europe'
 }
