STOPWORDS = [
        "'d",
        "'ll",
        "'m",
        "'re",
        "'s",
        "'ve",
        'a',
        'about',
        'above',
        'achieve',
        'acm',
        'across',
        'after',
        'afterwards',
        'again',
        'against',
        'ain',
        'all',
        'almost',
        'alone',
        'along',
        'already',
        'also',
        'although',
        'always',
        'am',
        'among',
        'amongst',
        'amoungst',
        'amount',
        'an',
        'and',
        'another',
        'any',
        'anyhow',
        'anyone',
        'anything',
        'anyway',
        'anywhere',
        'appproach',
        'approach',
        'are',
        'aren',
        "aren't",
        'around',
        'as',
        'at',
        'available',
        'back',
        'based',
        'be',
        'became',
        'because',
        'become',
        'becomes',
        'becoming',
        'been',
        'before',
        'beforehand',
        'behind',
        'being',
        'below',
        'beside',
        'besides',
        'between',
        'beyond',
        'bill',
        'both',
        'bottom',
        'but',
        'by',
        'ca',
        'call',
        'can',
        'cannot',
        'cant',
        'co',
        'combination',
        'computer',
        'con',
        'copyright',
        'could',
        'couldn',
        "couldn't",
        'couldnt',
        'cry',
        'current',
        'd',
        'de',
        'describe',
        'detail',
        'did',
        'didn',
        "didn't",
        'discuss',
        'do',
        'does',
        'doesn',
        "doesn't",
        'doing',
        'don',
        "don't",
        'done',
        'down',
        'due',
        'during',
        'each',
        'eg',
        'eight',
        'either',
        'eleven',
        'else',
        'elsewhere',
        'empty',
        'enough',
        'etc',
        'even',
        'ever',
        'every',
        'everyone',
        'everything',
        'everywhere',
        'except',
        'few',
        'fifteen',
        'fifty',
        'fill',
        'find',
        'fire',
        'first',
        'five',
        'for',
        'former',
        'formerly',
        'forty',
        'found',
        'four',
        'from',
        'front',
        'full',
        'further',
        'get',
        'give',
        'go',
        'had',
        'hadn',
        "hadn't",
        'has',
        'hasn',
        "hasn't",
        'hasnt',
        'have',
        'haven',
        "haven't",
        'having',
        'he',
        'hence',
        'her',
        'here',
        'hereafter',
        'hereby',
        'herein',
        'hereupon',
        'hers',
        'herself',
        'high',
        'him',
        'himself',
        'his',
        'how',
        'however',
        'hpc',
        'hundred',
        'i',
        'ie',
        'ieee',
        'if',
        'impact',
        'in',
        'inc',
        'increase',
        'indeed',
        'interest',
        'into',
        'is',
        'isn',
        "isn't",
        'it',
        "it's",
        'its',
        'itself',
        'just',
        'keep',
        'key',
        'kg',
        'km',
        'large',
        'last',
        'latter',
        'latterly',
        'least',
        'less',
        'll',
        'ltd',
        'm',
        'ma',
        'made',
        'make',
        'many',
        'may',
        'me',
        'meanwhile',
        'method',
        'might',
        'mightn',
        "mightn't",
        'mill',
        'mine',
        'more',
        'moreover',
        'most',
        'mostly',
        'move',
        'much',
        'must',
        'mustn',
        "mustn't",
        'my',
        'myself',
        "n't",
        'name',
        'namely',
        'need',
        'needn',
        "needn't",
        'neither',
        'never',
        'nevertheless',
        'new',
        'next',
        'nine',
        'no',
        'nobody',
        'none',
        'noone',
        'nor',
        'not',
        'nothing',
        'now',
        'nowhere',
        'n‘t',
        'n’t',
        'o',
        'of',
        'off',
        'often',
        'on',
        'once',
        'one',
        'only',
        'onto',
        'or',
        'other',
        'others',
        'otherwise',
        'our',
        'ours',
        'ourselves',
        'out',
        'over',
        'own',
        'paper',
        'part',
        'per',
        'perhaps',
        'please',
        'present',
        'propose',
        'proposed',
        'provide',
        'put',
        'quite',
        'rather',
        're',
        'real',
        'really',
        'regarding',
        'results',
        's',
        'same',
        'say',
        'see',
        'seem',
        'seemed',
        'seeming',
        'seems',
        'serious',
        'set',
        'several',
        'shan',
        "shan't",
        'she',
        "she's",
        'should',
        "should've",
        'shouldn',
        "shouldn't",
        'show',
        'side',
        'significant',
        'since',
        'sincere',
        'six',
        'sixty',
        'size',
        'so',
        'some',
        'somehow',
        'someone',
        'something',
        'sometime',
        'sometimes',
        'somewhere',
        'springer',
        'still',
        'such',
        'system',
        't',
        'take',
        'ten',
        'than',
        'that',
        "that'll",
        'the',
        'their',
        'theirs',
        'them',
        'themselves',
        'then',
        'thence',
        'there',
        'thereafter',
        'thereby',
        'therefore',
        'therein',
        'thereupon',
        'these',
        'they',
        'thick',
        'thin',
        'third',
        'this',
        'those',
        'though',
        'three',
        'through',
        'throughout',
        'thru',
        'thus',
        'time',
        'to',
        'today',
        'together',
        'too',
        'top',
        'toward',
        'towards',
        'twelve',
        'twenty',
        'two',
        'un',
        'under',
        'unless',
        'until',
        'up',
        'upon',
        'us',
        'use',
        'used',
        'using',
        'various',
        've',
        'verlag',
        'very',
        'via',
        'was',
        'wasn',
        "wasn't",
        'we',
        'well',
        'were',
        'weren',
        "weren't",
        'what',
        'whatever',
        'when',
        'whence',
        'whenever',
        'where',
        'whereafter',
        'whereas',
        'whereby',
        'wherein',
        'whereupon',
        'wherever',
        'whether',
        'which',
        'while',
        'whither',
        'who',
        'whoever',
        'whole',
        'whom',
        'whose',
        'why',
        'will',
        'with',
        'within',
        'without',
        'won',
        "won't",
        'work',
        'would',
        'wouldn',
        "wouldn't",
        'y',
        'yet',
        'you',
        "you'd",
        "you'll",
        "you're",
        "you've",
        'your',
        'yours',
        'yourself',
        'yourselves',
        '‘d',
        '‘ll',
        '‘m',
        '‘re',
        '‘s',
        '‘ve',
        '’d',
        '’ll',
        '’m',
        '’re',
        '’s',
        '’ve',
]
